# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cakes.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='cakes.proto',
  package='CAKES',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0b\x63\x61kes.proto\x12\x05\x43\x41KES\"\x19\n\x07\x45\x43\x44HKey\x12\x0e\n\x06pubkey\x18\x01 \x01(\x0c\"\x05\n\x03\x41\x63k\"/\n\x17IssueCertificateRequest\x12\x14\n\x0c\x45ncryptedCSR\x18\x02 \x01(\x0c\"P\n\x15IssueCertificateReply\x12\x1b\n\x13\x45ncryptedClientCert\x18\x01 \x01(\x0c\x12\x1a\n\x12\x45ncryptedCertChain\x18\x02 \x01(\x0c\x32\xb7\x01\n\x05\x43\x41KES\x12,\n\x0c\x43lientPubkey\x12\x0e.CAKES.ECDHKey\x1a\n.CAKES.Ack\"\x00\x12,\n\x0cServerPubkey\x12\n.CAKES.Ack\x1a\x0e.CAKES.ECDHKey\"\x00\x12R\n\x10IssueCertificate\x12\x1e.CAKES.IssueCertificateRequest\x1a\x1c.CAKES.IssueCertificateReply\"\x00\x62\x06proto3'
)




_ECDHKEY = _descriptor.Descriptor(
  name='ECDHKey',
  full_name='CAKES.ECDHKey',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='pubkey', full_name='CAKES.ECDHKey.pubkey', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=22,
  serialized_end=47,
)


_ACK = _descriptor.Descriptor(
  name='Ack',
  full_name='CAKES.Ack',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=49,
  serialized_end=54,
)


_ISSUECERTIFICATEREQUEST = _descriptor.Descriptor(
  name='IssueCertificateRequest',
  full_name='CAKES.IssueCertificateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='EncryptedCSR', full_name='CAKES.IssueCertificateRequest.EncryptedCSR', index=0,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=56,
  serialized_end=103,
)


_ISSUECERTIFICATEREPLY = _descriptor.Descriptor(
  name='IssueCertificateReply',
  full_name='CAKES.IssueCertificateReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='EncryptedClientCert', full_name='CAKES.IssueCertificateReply.EncryptedClientCert', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='EncryptedCertChain', full_name='CAKES.IssueCertificateReply.EncryptedCertChain', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=105,
  serialized_end=185,
)

DESCRIPTOR.message_types_by_name['ECDHKey'] = _ECDHKEY
DESCRIPTOR.message_types_by_name['Ack'] = _ACK
DESCRIPTOR.message_types_by_name['IssueCertificateRequest'] = _ISSUECERTIFICATEREQUEST
DESCRIPTOR.message_types_by_name['IssueCertificateReply'] = _ISSUECERTIFICATEREPLY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ECDHKey = _reflection.GeneratedProtocolMessageType('ECDHKey', (_message.Message,), {
  'DESCRIPTOR' : _ECDHKEY,
  '__module__' : 'cakes_pb2'
  # @@protoc_insertion_point(class_scope:CAKES.ECDHKey)
  })
_sym_db.RegisterMessage(ECDHKey)

Ack = _reflection.GeneratedProtocolMessageType('Ack', (_message.Message,), {
  'DESCRIPTOR' : _ACK,
  '__module__' : 'cakes_pb2'
  # @@protoc_insertion_point(class_scope:CAKES.Ack)
  })
_sym_db.RegisterMessage(Ack)

IssueCertificateRequest = _reflection.GeneratedProtocolMessageType('IssueCertificateRequest', (_message.Message,), {
  'DESCRIPTOR' : _ISSUECERTIFICATEREQUEST,
  '__module__' : 'cakes_pb2'
  # @@protoc_insertion_point(class_scope:CAKES.IssueCertificateRequest)
  })
_sym_db.RegisterMessage(IssueCertificateRequest)

IssueCertificateReply = _reflection.GeneratedProtocolMessageType('IssueCertificateReply', (_message.Message,), {
  'DESCRIPTOR' : _ISSUECERTIFICATEREPLY,
  '__module__' : 'cakes_pb2'
  # @@protoc_insertion_point(class_scope:CAKES.IssueCertificateReply)
  })
_sym_db.RegisterMessage(IssueCertificateReply)



_CAKES = _descriptor.ServiceDescriptor(
  name='CAKES',
  full_name='CAKES.CAKES',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=188,
  serialized_end=371,
  methods=[
  _descriptor.MethodDescriptor(
    name='ClientPubkey',
    full_name='CAKES.CAKES.ClientPubkey',
    index=0,
    containing_service=None,
    input_type=_ECDHKEY,
    output_type=_ACK,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ServerPubkey',
    full_name='CAKES.CAKES.ServerPubkey',
    index=1,
    containing_service=None,
    input_type=_ACK,
    output_type=_ECDHKEY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='IssueCertificate',
    full_name='CAKES.CAKES.IssueCertificate',
    index=2,
    containing_service=None,
    input_type=_ISSUECERTIFICATEREQUEST,
    output_type=_ISSUECERTIFICATEREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_CAKES)

DESCRIPTOR.services_by_name['CAKES'] = _CAKES

# @@protoc_insertion_point(module_scope)
